// ================================================================================
// (c) 2011 Altera Corporation. All rights reserved.
// Altera products are protected under numerous U.S. and foreign patents, maskwork
// rights, copyrights and other intellectual property laws.
//
// This reference design file, and your use thereof, is subject to and governed
// by the terms and conditions of the applicable Altera Reference Design License
// Agreement (either as signed by you, agreed by you upon download or as a
// "click-through" agreement upon installation andor found at www.altera.com).
// By using this reference design file, you indicate your acceptance of such terms
// and conditions between you and Altera Corporation.  In the event that you do
// not agree with such terms and conditions, you may not use the reference design
// file and please promptly destroy any copies you have made.
//
// This reference design file is being provided on an "as-is" basis and as an
// accommodation and therefore all warranties, representations or guarantees of
// any kind (whether express, implied or statutory) including, without limitation,
// warranties of merchantability, non-infringement, or fitness for a particular
// purpose, are specifically disclaimed.  By making this reference design file
// available, Altera expressly does not recommend, suggest or require that this
// reference design file be used in combination with any other product not
// provided by Altera.
// ================================================================================
//
// Software for the "Simulating Nios II Designs with Program Memory on External Flash
//    and SSRAM" Guide
// Jeff Wiencrot - 8/2/2011
//
// This program tests a range of addresses in SSRAM and signals possible write/read
// failures by turning on the LEDs.  Additionally, messages are sent to the console
// via the JTAG UART to confirm success or failure of each address.
//
// For more information, see "Simulating_Nios_II_Designs_with_Program_Memory_on_External_Flash_and_SSRAM.pdf"
//
// ================================================================================


#include <stdio.h>
#include "io.h"			// For IOWR and IORD
#include "system.h"		// Macros for SSRAM and LED address bases

// Define test addresses
#define START_ADDRESS	0x4000
#define END_ADDRESS		0x40FF

// Define LED states
#define LED_ON			0x0
#define LED_OFF			0xF
#define LED_SUCCESS		LED_OFF
#define LED_FAIL		LED_ON

int main()
{

	printf("Starting SSRAM test...\n");

	// Flash three times.
	IOWR(LED_PIO_BASE, 0, LED_ON);
	IOWR(LED_PIO_BASE, 0, LED_OFF);
	IOWR(LED_PIO_BASE, 0, LED_ON);
	IOWR(LED_PIO_BASE, 0, LED_OFF);
	IOWR(LED_PIO_BASE, 0, LED_ON);
	IOWR(LED_PIO_BASE, 0, LED_OFF);


	// Begin SSRAM test

	int i;

	for (i = START_ADDRESS; i <= END_ADDRESS; i ++)
	{
		printf("Address: %x", i);
		IOWR(EXT_SSRAM_BASE, i, (i & 0xF) );		// Write first byte of address as data to address

		if (IORD(EXT_SSRAM_BASE, i) == (i & 0xF) )	// Check data
		{
			printf(" - success\n");
			IOWR(LED_PIO_BASE, 0, LED_SUCCESS);
		}
		else
		{
			printf(" - fail\n");
			IOWR(LED_PIO_BASE, 0, LED_FAIL);
		}
		//donothingfor(100000);
		//usleep(1000000);
	}

	printf("Test complete.");

	return 0;
}
